@@ -18,6 +18,8 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#include <numeric>
+#include "algorithm.hpp"
 #include "InputBuffer.hpp"
 #include "InputReader.hpp"
 #include "PageRanges.hpp"
@@ -110,8 +112,7 @@ PageRanges PageRanges::filter (FilterFunc filterFunc) const {
 
 /** Returns the number of pages. */
 size_t PageRanges::numberOfPages () const {
-	size_t sum=0;
-	for (const auto &entry : *this)
-		sum += entry.second - entry.first + 1;
-	return sum;
+	return algo::accumulate(*this, 0, [](int sum, const Range &range) {
+		return sum + range.second - range.first + 1;
+	});
 }
