@@ -231,8 +231,7 @@ bool PsSpecialHandler::process (const string &prefix, istream &is, SpecialAction
 					fileType = FileType::BITMAP;
 			}
 			map<string,string> attr;
-			in.parseAttributes(attr, false);
-			imgfile(fileType, fname, attr);
+			imgfile(fileType, fname, in.parseAttributes(false));
 		}
 	}
 	else if (prefix == "ps::") {
@@ -565,7 +564,7 @@ void PsSpecialHandler::dviEndPage (unsigned, SpecialActions &actions) {
 			if (!isBaselineHorizontal)
 				Message::mstream() << "can't determine height, width, and depth due to non-horizontal baseline\n";
 			else {
-				const double bp2pt = 72.27/72.0;
+				constexpr double bp2pt = 72.27/72.0;
 				Message::mstream() <<
 					"width=" << XMLString(w*bp2pt) << "pt, "
 					"height=" << XMLString(h*bp2pt) << "pt, "
@@ -666,7 +665,7 @@ void PsSpecialHandler::closepath (vector<double>&) {
 
 void PsSpecialHandler::setblendmode (vector<double> &p) {
 	int mode = static_cast<int>(p[0]);
-	static const Opacity::BlendMode blendmodes[] = {
+	static constexpr Opacity::BlendMode blendmodes[] = {
 		Opacity::BM_NORMAL, Opacity::BM_MULTIPLY, Opacity::BM_SCREEN, Opacity::BM_OVERLAY,
 		Opacity::BM_SOFTLIGHT, Opacity::BM_HARDLIGHT, Opacity::BM_COLORDODGE, Opacity::BM_COLORBURN,
 		Opacity::BM_DARKEN, Opacity::BM_LIGHTEN, Opacity::BM_DIFFERENCE, Opacity::BM_EXCLUSION,
@@ -1061,11 +1060,7 @@ void PsSpecialHandler::shfill (vector<double> &params) {
 		const double &y1 = *it++;
 		const double &x2 = *it++;
 		const double &y2 = *it++;
-		bboxPath.moveto(x1, y1);
-		bboxPath.lineto(x2, y1);
-		bboxPath.lineto(x2, y2);
-		bboxPath.lineto(x1, y2);
-		bboxPath.closepath();
+		bboxPath.rect(x1, y1, x2, y2);
 		clip(std::move(bboxPath), false);
 	}
 	try {
