@@ -18,11 +18,12 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
-#include <algorithm>
 #include <fstream>
+#include <iterator>
 #include <list>
 #include <map>
 #include <sstream>
+#include "algorithm.hpp"
 #include "FileSystem.hpp"
 #include "utility.hpp"
 #include "XMLNode.hpp"
@@ -117,8 +118,8 @@ bool XMLElement::empty (bool ignoreWhitespace) const {
 	if (!_firstChild || !ignoreWhitespace)
 		return _firstChild == nullptr;
 
-	return all_of(begin(), end(), [](const XMLNode *node) {
-		return !node->toWSNode();
+	return algo::all_of(*this, [](const XMLNode *node) {
+		return node->toWSNode();
 	});
 }
 
@@ -136,8 +137,14 @@ void XMLElement::addAttribute (const string &name, double value) {
 }
 
 
+void XMLElement::addAttributes (const map<string, string> &attribs) {
+	for (auto &attr : attribs)
+		_attributes.emplace_back(attr);
+}
+
+
 void XMLElement::removeAttribute (const std::string &name) {
-	_attributes.erase(find_if(_attributes.begin(), _attributes.end(), [&](const Attribute &attr) {
+	_attributes.erase(algo::find_if(_attributes, [&](const Attribute &attr) {
 		return attr.name == name;
 	}));
 }
@@ -393,7 +400,7 @@ ostream& XMLElement::write (ostream &os) const {
 				ifstream ifs(fname, ios::binary);
 				if (ifs) {
 					os << '\n';
-					util::base64_copy(ifs, os, 200);
+					util::base64_encode(ifs, os, 200);
 					ifs.close();
 					if (!KEEP_ENCODED_FILES && !keep)
 						FileSystem::remove(fname);
@@ -440,7 +447,7 @@ const char* XMLElement::getAttributeValue (const std::string& name) const {
 
 
 XMLElement::Attribute* XMLElement::getAttribute (const string &name) {
-	auto it = find_if(_attributes.begin(), _attributes.end(), [&](const Attribute &attr) {
+	auto it = algo::find_if(_attributes, [&](const Attribute &attr) {
 		return attr.name == name;
 	});
 	return it != _attributes.end() ? &(*it) : nullptr;
@@ -448,7 +455,7 @@ XMLElement::Attribute* XMLElement::getAttribute (const string &name) {
 
 
 const XMLElement::Attribute* XMLElement::getAttribute (const string &name) const {
-	auto it = find_if(_attributes.begin(), _attributes.end(), [&](const Attribute &attr) {
+	auto it = algo::find_if(_attributes, [&](const Attribute &attr) {
 		return attr.name == name;
 	});
 	return it != _attributes.end() ? &(*it) : nullptr;
@@ -472,7 +479,7 @@ bool XMLElement::Attribute::inheritable () const {
 			"stroke-linecap", "stroke-linejoin", "stroke-miterlimit", "stroke-opacity", "stroke-width", "transform",
 			"visibility", "word-spacing", "writing-mode"
 	};
-	return binary_search(std::begin(names), std::end(names), name, [](const string &name1, const string &name2) {
+	return algo::binary_search(names, name, [](const string &name1, const string &name2) {
 		return name1 < name2;
 	});
 }
@@ -524,7 +531,7 @@ ostream& XMLCData::write (ostream &os) const {
 }
 
 
-void XMLCData::append (string &&str) {
+void XMLCData::append (string str) {
 	if (_data.empty())
 		_data = std::move(str);
 	else
